/*
 *  file      : lib_b2im/call.c
 *  project   : b2im
 *  copyright : (C) 2015 by BULIN Claude
 *
 *  This file is part of b2im project
 *
 *  b2im is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; version 3.
 *
 *  b2im is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */


#ifdef HAVE_CONFIG_H
	#include "../config.h"
#endif

#include <gtk/gtk.h>
#include <glib.h>

#include <pthread.h>
#include <sys/types.h>
#include <signal.h>
#include <sys/wait.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
// #include <fcntl.h>	// PROCESSUS NON-BLOQUANT

#include "lib.h"




#define CONV_MAX_CARS 1024


typedef struct {
	pid_t		code_fork_conv;				// Num Code Fork
	int			signal_numchildren_conv;	// the signal handler
	int			tube [ 2 ];					// for pipe
} CONV;

CONV conv;


// 
//
void call_sigchld( int signum )
{
	gint status;

	wait( &status );
    //  if there are still children waiting
    //  re-install the signal handler
	conv.signal_numchildren_conv --;
	if( conv.signal_numchildren_conv > 0 ) {
		// re-install the signal handler
		signal( SIGCHLD, call_sigchld );
	}
}
// 
//
int call_exec( gchar **args, pid_t *p, gint p_output )
{
	gchar	**ptr = (gchar **)args;
	// int	status;

	conv.signal_numchildren_conv = 0;
	if( pipe (conv.tube ) != 0 ) {
		fprintf( stderr, "error: pipe\n" );
		exit( 1 );
	}
	if( (*p = fork()) == 0 ) {
		dup2( conv.tube [ 1 ], p_output );
		close( conv.tube [ 1 ] );
		execvp( (gchar *)*(ptr+0), ptr );
		fprintf( stderr, "error: exec" );
		exit( 2 );
	}
	conv.signal_numchildren_conv ++;
	signal( SIGCHLD, call_sigchld );
	close( conv.tube [ 1 ] );

	// LE FICHIER EST OUVERT EN MODE « NON-BLOQUANT »
	// status = fcntl( conv.tube [ 0 ], F_GETFL );
	// status = fcntl( conv.tube [ 0 ], F_SETFL, status | O_NONBLOCK );

	return( conv.tube [ 0 ] );
}
// 
//
void call_with_( gint p_output, gchar **args )
{
	gint	fd;
	gint	size;
	gint	pos = 0;
	gchar	buf [ CONV_MAX_CARS + 10 ];

	fd = call_exec( args, &conv.code_fork_conv, p_output ); // STDERR_FILENO  STDOUT_FILENO  STDIN_FILENO
	do {
		pos = -1;
		do {
			pos++;
			if( pos >= CONV_MAX_CARS ) {
				printf( "pos(%d) >= CONV_MAX_CARS(%d)\n", pos, CONV_MAX_CARS );
				pos --;
			}
			size = read( fd, &buf[pos], 1 );

		} while( (buf[pos] != '\b') && (buf[pos] != '\r') && (buf[pos] != '\n') && (size > 0) );
		pos ++;
		buf[ pos ] = '\0';
		buf [ 0 ] = '\0';
		buf [ 1 ] = '\0';
		buf [ 2 ] = '\0';

	} while( size != 0 );

	close( fd );
	conv.code_fork_conv = -1;
}
// 
// 
void call_with_args( gint p_output, gchar *p_text, ... )
{
	va_list	VaList;
	gchar	*Ptr = NULL;
	gchar	**PtrTabArgs = (gchar **)g_malloc0( sizeof(gchar **) * 50 );
	gint	Cpt = 0;
	
	// Get arguments
	PtrTabArgs[ Cpt ++ ] = g_strdup( p_text );
	va_start( VaList, p_text );
	while( NULL != ( Ptr = va_arg( VaList, gchar * )))
		 PtrTabArgs[ Cpt ++ ] = g_strdup( Ptr );
	PtrTabArgs[ Cpt ] = NULL;
	va_end( VaList );
	
	// Print arguments
	/*
	printf("\n" );
	printf("p_output = %d\n", p_output );
	for( Cpt = 0; PtrTabArgs[ Cpt ] != NULL; Cpt ++ )
		printf("%2d %s\n", Cpt, PtrTabArgs[ Cpt ] );	
	printf("\n" );
	*/
	// Call external function
	call_with_( p_output, PtrTabArgs );
	
	// Remove arguments
	for( Cpt = 0; PtrTabArgs[ Cpt ] != NULL; Cpt ++ ) {
		g_free( PtrTabArgs[ Cpt ] );
		PtrTabArgs[ Cpt ] = NULL;
	}
	g_free( PtrTabArgs );
	PtrTabArgs = NULL;
}







