/*
 *  file      : src/ntb_to_home.c
 *  project   : b2im
 *  copyright : (C) 2015 by BULIN Claude
 * 
 *  This file is part of b2im project
 * 
 *  b2im is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; version 3.
 * 
 *  b2im is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */


#ifdef HAVE_CONFIG_H
	#include "../config.h"
#endif
#include <gtk/gtk.h>
#include <glib/gstdio.h>
#include <glib.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#ifdef ENABLE_STATIC_LIBRARY
	#include "../lib_b2im/lib.h"
#endif

#include "global.h"
#include "dragNdrop.h"
#include "configuser.h"




typedef struct {
	GtkWidget	*adr_textview_home;
	GtkWidget	*adr_textview_config;
	GtkWidget	*adr_textview_local_share;
} VAR_NTBHOME;

VAR_NTBHOME VarNtbHome;



// 
// 
void NtbHome_add_datas_to_( GtkWidget *widget, GSList *p_list )
{
	GSList		*gs_List = p_list;
	gchar		*Ptr = NULL;
	GtkWidget	*adr_textview = NULL;
	
	if( widget == GTK_WIDGET(GLADE_GET_OBJECT("textview_add_to_home")) )					adr_textview = VarNtbHome.adr_textview_home;
	else if( widget == GTK_WIDGET(GLADE_GET_OBJECT("textview_add_to_home_config")) )		adr_textview = VarNtbHome.adr_textview_config;
	else if( widget == GTK_WIDGET(GLADE_GET_OBJECT("textview_add_to_home_local_share")) )	adr_textview = VarNtbHome.adr_textview_local_share;
	
	while( gs_List ) {
		if( (Ptr = (gchar *)gs_List->data) != NULL ) {
			libutils_set_text_to_textview( adr_textview, Ptr );
		}
		gs_List = g_slist_next( gs_List );
	}
}
// 
// 
static void NtbHome_drag_data_received(
					GtkWidget        *widget,
					GdkDragContext   *context,
					gint              x,
					gint              y,
					GtkSelectionData *data,
					guint             info,
					guint             time,
					gpointer          user_data)
{
	if (gtk_drag_get_source_widget(context) != widget) {
		dragndrop_list_drag_data( widget, (gchar*)gtk_selection_data_get_data( data ));
	}
}
// 
// 
void on_textview_add_to_home_realize( GtkWidget *widget, gpointer user_data )
{
	VarNtbHome.adr_textview_home = widget;
	gtk_drag_dest_set (GTK_WIDGET (widget),
			   GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT,
			   drag_types,
			   n_drag_types,
			   GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_DEFAULT
			   );
	g_signal_connect(G_OBJECT(widget), "drag-data-received", G_CALLBACK(NtbHome_drag_data_received), NULL);
}
// 
// 
void on_textview_add_to_home_config_realize( GtkWidget *widget, gpointer user_data )
{
	VarNtbHome.adr_textview_config = widget;
	gtk_drag_dest_set (GTK_WIDGET (widget),
			   GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT,
			   drag_types,
			   n_drag_types,
			   GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_DEFAULT
			   );
	g_signal_connect(G_OBJECT(widget), "drag-data-received", G_CALLBACK(NtbHome_drag_data_received), NULL);
}
// 
// 
void on_textview_add_to_home_local_share_realize( GtkWidget *widget, gpointer user_data )
{
	VarNtbHome.adr_textview_local_share = widget;
	gtk_drag_dest_set (GTK_WIDGET (widget),
			   GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT,
			   drag_types,
			   n_drag_types,
			   GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_DEFAULT
			   );
	g_signal_connect(G_OBJECT(widget), "drag-data-received", G_CALLBACK(NtbHome_drag_data_received), NULL);
}

void NtbHome_restore_datas_to_home_( void )
{
	FILE	*fp;
	gchar	*PathDesktop = NtbDesktop_get_first_name_desktop_actif();
	
	printf("%s*-*-> Restore datas in %s/%s/%s-overlay%s\n",
			BOLD_YELLOW,
			Config.path_folder_manjaro_profiles,
			PathDesktop,
			PathDesktop,
			RESET
			);

	fp = fopen( "/tmp/run_restore_to_home.sh", "w" );
	fprintf( fp, "#!/bin/bash\n" );
	fprintf( fp, "#\n\n" );
	fprintf( fp, "# Restore directory\n" );
	// libutils_remove_rep( gchar *path_tmprep );
	fprintf( fp, "rm -rf %s/%s/%s-overlay\n\n", 
			Config.path_folder_manjaro_profiles,
			PathDesktop,
			PathDesktop
			); 
	fprintf( fp, "mv %s/%s/SAVE_%s-overlay %s/%s/%s-overlay\n\n", 
			Config.path_folder_manjaro_profiles,
			PathDesktop,
			PathDesktop,
			Config.path_folder_manjaro_profiles,
			PathDesktop,
			PathDesktop
			); 
	fclose( fp );
	g_free( PathDesktop );
	PathDesktop = NULL;

	g_chmod( "/tmp/run_restore_to_home.sh", 0700 );   
	// system( "/tmp/run_restore_to_home.sh" );
	call_with_args( 
				STDOUT_FILENO,
				"/tmp/run_restore_to_home.sh",
				NULL
				);
	g_unlink( "/tmp/run_restore_to_home.sh" );
}

void NtbHome_save_datas_to_home_( void )
{
	gchar	**Larrbuf;
	gint	Cpt;
	FILE	*fp;
	gchar	*PathDesktop = NULL;
	gchar	*PathDest = NULL;
	
	printf("%s*-*-> Save datas to $HOME, $HOME/.config, $HOME/.local/share%s\n", BOLD_YELLOW, RESET );

	// textview_add_to_home
	PathDesktop = NtbDesktop_get_first_name_desktop_actif();
	PathDest = g_strdup_printf("%s/%s/%s-overlay/etc/skel",
			Config.path_folder_manjaro_profiles,
			PathDesktop,
			PathDesktop
			); 
	
	Larrbuf = libutils_get_textview( GTK_WIDGET(GLADE_GET_OBJECT("textview_add_to_home")) );
	fp = fopen( "/tmp/run_save_to_home.sh", "w" );
	fprintf( fp, "#!/bin/bash\n" );
	fprintf( fp, "#\n\n" );

	fprintf( fp, "# Save directory\n" );
	fprintf( fp, "cp -a %s/%s/%s-overlay %s/%s/SAVE_%s-overlay\n\n", 
			Config.path_folder_manjaro_profiles,
			PathDesktop,
			PathDesktop,
			Config.path_folder_manjaro_profiles,
			PathDesktop,
			PathDesktop
			); 
	
	fprintf( fp, "# Save datas to: $HOME\n" );
	for( Cpt = 0; Larrbuf[ Cpt ]; Cpt++ ) {
		if( *Larrbuf[ Cpt ] != '\0' ) {
			
			fprintf( fp, "cp " );
			if( TRUE == g_file_test( (gchar *)Larrbuf[ Cpt ], G_FILE_TEST_IS_DIR )) fprintf( fp, "-a " );
			fprintf( fp, "%s %s\n", (gchar *)Larrbuf[ Cpt ], PathDest );
		}
	}
	fprintf( fp, "\n" );
	g_strfreev( Larrbuf );
	Larrbuf = NULL;

	// textview_add_to_home_config
	g_free( PathDest );
	PathDest = NULL;
	PathDest = g_strdup_printf("%s/%s/%s-overlay/etc/skel/.config",
			Config.path_folder_manjaro_profiles,
			PathDesktop,
			PathDesktop
			); 
	Larrbuf = libutils_get_textview( GTK_WIDGET(GLADE_GET_OBJECT("textview_add_to_home_config")) );
	fprintf( fp, "\n" );
	fprintf( fp, "mkdir -p %s\n", PathDest );
	fprintf( fp, "# Save datas to: $HOME/.config\n" );
	for( Cpt = 0; Larrbuf[ Cpt ]; Cpt++ ) {
		if( *Larrbuf[ Cpt ] != '\0' ) {
			
			fprintf( fp, "cp " );
			if( TRUE == g_file_test( (gchar *)Larrbuf[ Cpt ], G_FILE_TEST_IS_DIR )) fprintf( fp, "-a " );
			fprintf( fp, "%s %s\n", (gchar *)Larrbuf[ Cpt ], PathDest );
		}
	}
	fprintf( fp, "\n" );
	g_strfreev( Larrbuf );
	Larrbuf = NULL;

	// textview_add_to_home_local_share
	g_free( PathDest );
	PathDest = NULL;
	PathDest = g_strdup_printf("%s/%s/%s-overlay/etc/skel/.local/share",
			Config.path_folder_manjaro_profiles,
			PathDesktop,
			PathDesktop
			); 
	Larrbuf = libutils_get_textview( GTK_WIDGET(GLADE_GET_OBJECT("textview_add_to_home_local_share")) );
	fprintf( fp, "\n" );
	fprintf( fp, "mkdir -p %s\n", PathDest );
	fprintf( fp, "# Save datas to: $HOME/.local/share\n" );
	for( Cpt = 0; Larrbuf[ Cpt ]; Cpt++ ) {
		if( *Larrbuf[ Cpt ] != '\0' ) {
			
			fprintf( fp, "cp " );
			if( TRUE == g_file_test( (gchar *)Larrbuf[ Cpt ], G_FILE_TEST_IS_DIR )) fprintf( fp, "-a " );
			fprintf( fp, "%s %s\n", (gchar *)Larrbuf[ Cpt ], PathDest );
		}
	}
	fprintf( fp, "\n" );
	g_strfreev( Larrbuf );
	Larrbuf = NULL;
	
	fclose( fp );
	g_chmod( "/tmp/run_save_to_home.sh", 0700 );   
	
	g_free( PathDesktop );
	PathDesktop = NULL;
	g_free( PathDest );
	PathDest = NULL;
	
	// system( "/tmp/run_save_to_home.sh" );
	call_with_args(
				STDOUT_FILENO,
				"/tmp/run_save_to_home.sh",
				NULL
				);
	g_unlink( "/tmp/run_save_to_home.sh" );
}














